#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Testing module for most of GUI."""

import pytest
from torrentfile import TorrentFile
from torrentfile import magnet

from tests import dir1, rmpath, tempfile, wind, MockEvent, ttorrent, dir2


@pytest.fixture(scope="module")
def torrent(dir1):
    """Creates torrent pytest fixture."""
    args = {
        "path": dir1,
        "source": "source",
        "announce": ["https://tracker1.com/announce"],
        "piece_length": "16",
        "comment": "some comment",
    }
    torrent = TorrentFile(**args)
    outfile, meta = torrent.write()
    yield outfile, meta
    rmpath(outfile)


def test_fixture():
    """Testing pytest fixtures."""
    assert [dir1, tempfile, wind, ttorrent, dir2]


def test_create_magnet(wind, torrent):
    """Test creating Magnet URI from a torrent file path."""
    window, app = wind
    outfile, _ = torrent
    tab = window.central.magnetWidget
    window.central.setCurrentWidget(tab)
    tab.metafile_input.setText(str(outfile))
    tab.submit_button.click()
    out = tab.output.text()
    app.processEvents()
    assert out == magnet(outfile)


def test_create_magnet_method(wind, torrent):
    """Test creating Magnet URI from a torrent file path."""
    window, app = wind
    outfile, _ = torrent
    tab = window.central.magnetWidget
    window.central.setCurrentWidget(tab)
    tab.file_button.select_metafile(filename=outfile)
    out = tab.output.text()
    app.processEvents()
    assert out == magnet(outfile)


def test_magnet_accept_method(wind, ttorrent):
    """Test drag enter event on editor widget."""
    window, app = wind
    magnet = window.central.magnetWidget
    magnet.window.central.setCurrentWidget(magnet)
    app.processEvents()
    event = MockEvent(ttorrent)
    assert magnet.dragEnterEvent(event)
    event = MockEvent(None)
    assert not magnet.dragEnterEvent(event)


def test_magnet_move_event(wind, ttorrent):
    """Test move event on magnet widget."""
    window, app = wind
    magnet = window.central.magnetWidget
    magnet.window.central.setCurrentWidget(magnet)
    app.processEvents()
    event = MockEvent(ttorrent)
    assert magnet.dragMoveEvent(event)
    event = MockEvent(None)
    assert not magnet.dragMoveEvent(event)


def test_magnet_drop_event(wind, ttorrent):
    """Test drop event on editor widget."""
    window, app = wind
    magnet = window.central.magnetWidget
    magnet.window.central.setCurrentWidget(magnet)
    app.processEvents()
    event = MockEvent(ttorrent)
    assert magnet.dropEvent(event)


def test_magnet_drop_false(wind):
    """Test drop event on editor widget returns None."""
    window, app = wind
    magnet = window.central.magnetWidget
    magnet.window.central.setCurrentWidget(magnet)
    app.processEvents()
    event = MockEvent(None)
    assert not magnet.dropEvent(event)



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Module for stylesheets."""

import os
from urllib.request import pathname2url as path2url

arrow = os.path.join(os.environ['ASSETS'], 'icons', 'arrow-down.png')


dark_theme = """
* {
    background-color: #19232D;
    color: #FFFFFF;
    padding: 0px;
    margin: 0px;
    border-width: 0px;
    border-style: solid;
}

QMenu,
QTreeWidget,
QTreeWidget::item,
QTableWidget,
QTableWidget::item,
QHeaderView::section,
QTableView QTableCornerButton::section,
QLineEdit,
QTextEdit,
QComboBox,
QComboBox:editable,
QComboBox QAbstractItemView,
QProgressBar,
QScrollBar {
    border-width: 0px;
    border-style: solid;
    border-radius: 0px;
    padding: 0px;
    margin: 0px;
}

QHeaderView::section,
QTableView QTableCornerButton::section,
QLineEdit,
QTextEdit,
QComboBox:editable,
QComboBox,
QComboBox QAbstractItemView,
QProgressBar,
QScrollBar {
    border-color: #f61;
    selection-background-color: #2f4f4f;
}

QLabel,
QLabel:disabled {
    border-width: 1px;
    background-color: transparent;
    padding: 0px;
    border-color: transparent;
    selection-background-color: #2f4f4f;
}

QMainWindow::separator {
    background: #000;
    border-width: 0px;
}

QMainWindow::separator:hover {
    background: transparent;
}

QWidget {
    background-color: #39434d;
    border-width: 0px;
}

QAbstractScrollArea {
    background-color: #19232D;
    border: 1px solid #f61;
    border-radius: 4px;
    padding: 2px;
    color: #E0E1E3;
}

QAbstractScrollArea:disabled {
    color: #9DA9B5;
}

QScrollArea QWidget QWidget:disabled {
    background-color: #19232D;
}

QScrollBar:horizontal {
    height: 16px;
    margin: 2px 16px 2px 16px;
    border: 1px solid #455364;
    border-radius: 4px;
    background-color: #19232D;
}

QScrollBar:vertical {
    background-color: #19232D;
    width: 16px;
    margin: 16px 2px 16px 2px;
    border: 1px solid #455364;
    border-radius: 4px;
}

QScrollBar::handle:horizontal {
    background-color: #60798B;
    border: 1px solid #455364;
    border-radius: 4px;
    min-width: 8px;
}

QScrollBar::handle:horizontal:hover {
    background-color: #346792;
    border: #346792;
    border-radius: 4px;
    min-width: 8px;
}

QScrollBar::handle:horizontal:focus {
    border: 1px solid #1A72BB;
}

QScrollBar::handle:vertical {
    background-color: #60798B;
    border: 1px solid #455364;
    min-height: 8px;
    border-radius: 4px;
}

QScrollBar::handle:vertical:hover {
    background-color: #346792;
    border: #346792;
    border-radius: 4px;
    min-height: 8px;
}

QScrollBar::handle:vertical:focus {
    border: 1px solid #1A72BB;
}

QScrollBar::add-line:horizontal {
    margin: 0px 0px 0px 0px;
    height: 12px;
    width: 12px;
    subcontrol-position: right;
    subcontrol-origin: margin;
}

QScrollBar::add-line:horizontal:hover, QScrollBar::add-line:horizontal:on {
    height: 12px;
    width: 12px;
    subcontrol-position: right;
    subcontrol-origin: margin;
}

QScrollBar::add-line:vertical {
    margin: 3px 0px 3px 0px;
    height: 12px;
    width: 12px;
    subcontrol-position: bottom;
    subcontrol-origin: margin;
}

QScrollBar::add-line:vertical:hover, QScrollBar::add-line:vertical:on {
    height: 12px;
    width: 12px;
    subcontrol-position: bottom;
    subcontrol-origin: margin;
}

QScrollBar::sub-line:horizontal {
    margin: 0px 3px 0px 3px;
    height: 12px;
    width: 12px;
    subcontrol-position: left;
    subcontrol-origin: margin;
}

QScrollBar::sub-line:horizontal:hover, QScrollBar::sub-line:horizontal:on {
    height: 12px;
    width: 12px;
    subcontrol-position: left;
    subcontrol-origin: margin;
}

QScrollBar::sub-line:vertical {
    margin: 3px 0px 3px 0px;
    height: 12px;
    width: 12px;
    subcontrol-position: top;
    subcontrol-origin: margin;
}

QScrollBar::sub-line:vertical:hover, QScrollBar::sub-line:vertical:on {
    height: 12px;
    width: 12px;
    subcontrol-position: top;
    subcontrol-origin: margin;
}

QTabWidget::pane {
    border-top: 2px solid black;
}

QTabWidget::tab-bar {
    left: 5px;
}

QTabBar::tab {
    border-style: outset;
    border-color: #f61;
    border-width: 3px;
    font-size: 10pt;
    color: #e9e7e6;
    padding-left: 8px;
    padding-right: 8px;
    margin-right: 2px;
    padding-bottom: 2px;
    background-color: #444;
}

QTabBar::tab:hover {
    color: #ffffff;
}

QTabBar::tab:selected, QTabBar::tab:last:selected {
    border-color: #e76500;
    font-style: bold;
    border-width: 2px;
    border-style: inset;
    color: #ffffff;
    padding-left: 8px;
    padding-right: 8px;
    padding-bottom: 0px;
    margin-right: 2px;
    background-color: #333;
}

QPushButton{
    margin-top: 3px;
    margin-bottom: 3px;
    margin-left: 8px;
    margin-right: 8px;
    border-style: outset;
    border-top-color: #e18133;
    border-right-color: #e67f2f;
    border-left-color: #e67f2f;
    border-bottom-color: #e67e22;
    border-radius: 8px;
    border-width: 3px;
    padding-top: 4px;
    padding-bottom: 4px;
    font: 13pt bold;
    color: #dedede;
    background-color: #111;
}

QPushButton:hover{
    border-style: inset;
    border-width: 3px;
    color: #efefefef;
    background-color: #000;
}

QPushButton:pressed{
    border-style: inset;
    border-bottom-width: 3px;
    color: #ffffff;
    background-color: #112;
}

QPushButton:disabled{
    border-style: solid;
    border-bottom-width: 1px;
    border-style: solid;
    color: #bbb;
    padding-bottom: 1px;
    background-color: #444;
}

QToolButton {
    font-size: 9pt;
    border-style: outset;
    border-color: #e67e22;
    border-width: 3px;
    border-radius: 8px;
    color: #FFF;
    padding: 5px;
    padding-top: 3px;
    background-color: #112;
}

QToolButton:hover {
    border-style: inset;
    margin-bottom: 1px;
    padding: 4px;
    background-color: #000;
}

QRadioButton {
    background-color: transparent;
    font-size: 11pt;
    border: transparent;
    padding-top: 2px;
    margin-top: 2px;
    margin-bottom: 2px;
    padding-bottom: 4px;
    color: #fff;
}

QRadioButton::indicator::unchecked {
    border: 1px inset #f73;
    border-radius: 6px;
    background-color:  #323232;
    width: 13px;
    height: 13px;
}

QRadioButton::indicator::unchecked:hover {
    border-radius: 5px;
    background-color:  #323232;
    width: 13px;
    height: 13px;
}

QRadioButton::indicator::checked {
    border: 1px inset #d73;
    border-radius: 5px;
    background-color: #ff5;
    width: 13px;
    height: 13px;
}

QCheckBox {
    color: #fff;
    padding: 4px;
    font-size: 11pt;
    border-color: transparent;
}

QCheckBox:disabled {
    color: #808086;
    padding: 6px;
}

QCheckBox::indicator:checked {
    height: 13px;
    width: 13px;
    border-style:solid;
    border-width: 2px;
    border-color: #e67e22;
    color: #a9b7c6;
    background-color: #d63d12;
}

QCheckBox::indicator:unchecked {
    height: 11px;
    width: 11px;
    border-style:solid;
    border-width: 2px;
    border-color: #e67e22;
    color: #a9b7c6;
    background-color: transparent;
}

QTreeWidget {
    border: 1px solid #f61;
    background-color: #5a5a5a;
    font: 8pt;
    color: #eeeeee;
}

QTreeWidget::item::selected {
    color: #FFFFFF;
    font-size: 9pt;
    border-color: transparent;
}

QTreeWidget::item::hover {
    font-size: 9pt;
    color: #CFF8DC;
    border-color: transparent;
}

QTreeWidget::indicator::checked {
    background-color: #ffffff;
    border: 1px solid #536D79;
}

QTreeWidget::indicator::unchecked {
    background-color: #ffffff;
    border: 1px solid #536D79;
}

QTreeWidget QHeaderView::section {
    background-color: black;
    color: white;
}

QTreeWidget QHeaderView::section {
    border: 1px solid #e57e22;
    background-color: black;
    color: white;
}

QTreeWidget QProgressBar {
    color: black;
    background-color: #7a7a7a;
    border: 1px solid black;
    border-radius: 3px;
    margin-left: 2px;
    margin-right: 2px;
    text-align: center;
}

QTreeWidget QProgressBar::chunk {
    background-color: #3ae1de;
    margin-left: .5px;
    margin-right: 0px;
    margin-top: 0px;
    margin-bottom: 0px;
    border-radius: 3px;
    width: 16px;
}

QTableWidget {
    background-color: #5a5a5a;
    color: #ccc;
    border-color: #e57e22;
    border-width: 2px;
    border-style: ridge;
    border-radius: 4px;
    margin: 10px;
    font-size: 11pt;
    selection-background-color: #3a3a3a;
    selection-color: #fff;
    gridline-color: #ac4a02;
}

QTableWidget::item:selected:hover {
    background-color: #7c7d7b;
}

QTableWidget::item:hover {
    background-color: #225;
}

QTableWidget QTableCornerButton::section{
    background-color: black;
    border: 2px brown solid;
}

QHeaderView::section {
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0,
                    y2: 1, stop:0 #54585b, stop:1 #393c3e);
    color: #bbbbbb;
    padding: 1px 2px 1px 4px;
    border: 1px solid #323232;
    border-top-width: 0;
    border-left-color: #5e6163;
    border-right-color: #2a2c2d;
}
QHeaderView::section:hover {
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0,
                    y2: 1, stop:0 #64686b, stop:1 #494c4e);
    border-bottom-color: #424242;
}
QHeaderView::section:first {
    border-left-width: 0;
}
QHeaderView::section:last {
    border-right-width: 0;
}
QHeaderView::section:checked {
    color: #fff;
    background-color: #222;
}
QMenuBar {
    background-color: #322;
    padding: 2px;
    margin-bottom: 2px;
    border-bottom: 2px solid #555555;
}
QMenuBar::item:pressed {
    background-color: #4b6eaf;
}
QMenuBar::item:selected:!pressed {
    background-color: #585b5d;
}
QMenu {
    border: 1px solid #2d2d2d;
}
QMenu::item:disabled {
    color: #999999;
}
QMenu::item:selected {
    background-color: #4b6eaf;
}
QMenu::icon {
    border: 0px solid transparent;
    background-color: transparent;
}
QMenu::icon:checked {
    background-color: blue;
    border: 1px inset red;
    position: absolute;
    top: 1px;
    right: 1px;
    bottom: 1px;
    left: 1px;
}
QMenu::separator {
    height: 2px;
    background-color: qlineargradient(x1: 0, y1: 0, x2: 0,
                    y2: 1, stop:0 #282a2b, stop:1 #45484b);
    margin: 0 1px;
}
QMenu::indicator {
    width: 13px;
    height: 13px;
    background-color: blue;
}

QDialog {
    background-color:#000000;
}

QLineEdit {
    padding: 4px;
    border-width: 1px;
    border-style: groove;
    margin-top: 3px;
    margin-bottom: 3px;
    font: 9pt;
    background-color: #19232D;
}

QLabel {
    padding: 2px;
    padding-top: 4px;
    padding-bottom: 4px;
    font-size: 10pt;
    font-weight: bold;
}

QStatusBar {
    border: 2px groove black;
}

QComboBox {
    border: 2px solid #f73;
    padding: 1px 1px 1px 3px;
    font-size: 10pt;
    min-width: 2em;
}
QComboBox:on {
    padding-top: 3px;
    padding-left: 4px;
}
QComboBox::drop-down {
    width: 20px;
    background: #555;
    padding: 0px 2px 0px 2px;
    border-left-width: 2px;
    border-left-color: #311;
    border-left-style: outset;
}
QComboBox::down-arrow:on {
    top: 1px;
}
QComboBox QAbstractItemView {
    border: 1px solid #444;
    selection-background-color: lightgray;
}
QComboBox::down-arrow {
    image: url(%s);
}
""" % path2url(
    str(arrow)
)

light_theme = """
* {
    padding: 0px;
    margin: 0px;
    border: transparent 0px solid;
    border-radius: 0px;
    font-size: 10pt;
}
QLineEdit,
QTextEdit,
QListWidget,
QTableWidget,
QComboBox,
QTreeWidget {
    color: black;
    background-color: #000;
    border-color: black;
    border-width: 1px;
    border-radius: 2px;
    border-style: solid;
}
QTabBar::tab
QLabel,
QCheckBox,
QRadioButton,
QPushButton,
QToolButton {
    font-weight: bold;
    font-size: 10pt;
}
QPushButton,
QToolButton {
    border-style: outset;
    border-width: 3px;
    border-color: #555;
    border-radius: 8px;
    background-color: #efeccf;
}
QPushButton::hover,
QToolButton::hover {
    background-color: #dfdcef;
}
QPushButton:pressed,
QToolButton:pressed {
    border-style: inset;
    border-color: #333;
    background-color: #efeccc;
}
QDialog,
QWidget {
    background-color: #eae7e9;
}
QTabWidget::pane {
    border-top: 0px solid
}
QTabWidget::tab-bar {
    left: 5px;
}
QTabBar {
    border-color: transparent;
}
QTabBar::tab {
    border-style: outset;
    border-color: black;
    font-size: 10pt;
    border-right-width: 1px;
    border-left-width: 1px;
    border-top-width: 1px;
    border-bottom-color: transparent;
    border-bottom-width: 0px;
    color: black;
    padding-left: 8px;
    padding-right: 8px;
    margin-right: 2px;
    padding-bottom: 2px;
    background-color: #d1d0c9;
}
QTabBar::tab:hover {
    color: #00F;
}
QTabBar::tab:selected, QTabBar::tab:last:selected {
    border-color: black;
    font-style: bold;
    border-width: 2px;
    border-bottom-width: 0px;
    border-style: inset;
    color: black;
    padding-left: 8px;
    padding-right: 8px;
    padding-bottom: 0px;
    margin-right: 2px;
    background-color: #eae7e9;
}
QAbstractScrollArea,
QCombobox QAbstractItemView {
    background-color: #eee;
    border: 1px solid #444;
    border-radius: 4px;
    padding: 2px;
    color: #000;
}
"""

infoLineEdit = """
QLineEdit {
    background-color: transparent;
    color: #FFF;
    border-radius: 0px;
    border-width: 2px;
    font-size: 11pt;
    padding: 0px;
    margin: 0px;
    border-left-color: transparent;
    border-right-color: transparent;
    border-top-color: transparent;
}
"""

infoLineEditLight = """
QLineEdit {
    border-radius: 0px;
    border-width: 2px;
    color: black;
    font-size: 11pt;
    padding: 0px;
    margin: 0px;
    border-left-color: transparent;
    border-right-color: transparent;
    border-top-color: transparent;
    border-bottom-color: black;
}
"""

pushButtonEdit = """
QPushButton {
    padding: 3px;
    margin: 0px;
    font-size: 9pt;
}
"""

table_styles = {
    "LineEdit": """
    QLineEdit {
        padding: 0px;
        margin: 0px;
        padding: 0px;
    }
    """,
    "ComboBox": """
    QComboBox {
        font-size: 12pt;
        margin: 0px 0px 0px 0px;
        border-width: 1px;
        padding: 0px 0px 0px 0px;
        min-height: 19px;
    }
    """,
    "button": """
    QToolButton {
        margin: 2px;
        padding: 2px;
        border-radius: 12px;
    }
    """
}



##############################################################################
# Copyright 20** AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Application version id tag."""

import os


def _conf():
    """Create some enviornment variables."""
    parent = os.path.dirname(__file__)
    top = os.path.dirname(parent)
    assets = os.path.join(top, 'assets')
    path = os.path.relpath(assets, ".")
    return path


ASSETS = str(_conf())
os.environ['ASSETS'] = ASSETS

__version__ = "0.3.2"



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Module for testing procedures on Check Tab."""

import os
from pathlib import Path

import pytest

from tests import dir1, dir2, rmpath, tempfile, ttorrent, wind
from torrentfileQt.checkTab import ProgressBar, TreePieceItem, TreeWidget


def test_fixture():
    """Test Fixtures."""
    assert dir1 and dir2 and ttorrent and wind


def test_missing_files_check(dir2, ttorrent, wind):
    """Test missing files checker proceduire."""
    window, _ = wind
    checktab = window.central.checkWidget
    window.central.setCurrentWidget(checktab)
    dirpath = Path(dir2)
    for item in dirpath.iterdir():
        if item.is_file():
            os.remove(item)
    checktab.fileInput.setText(ttorrent)
    checktab.searchInput.setText(dir2)
    checktab.checkButton.click()
    assert checktab.treeWidget.topLevelItemCount() > 0


def test_shorter_files_check(wind, ttorrent, dir2):
    """Test missing files checker proceduire."""
    window, _ = wind
    checktab = window.central.checkWidget
    dirpath = Path(dir2)
    window.central.setCurrentWidget(checktab)

    def shortenfile(item):
        """Shave some data off the end of file."""
        temp = bytearray(2 ** 19)
        with open(item, "rb") as fd:
            fd.readinto(temp)
        with open(item, "wb") as fd:
            fd.write(temp)

    if os.path.exists(dirpath):
        for item in dirpath.iterdir():
            if item.is_file():
                shortenfile(item)
    checktab.fileInput.setText(ttorrent)
    checktab.searchInput.setText(dir2)
    checktab.checkButton.click()
    assert checktab.treeWidget.topLevelItemCount() > 0


def test_check_tab(wind, ttorrent, dir1):
    """Test checker procedure."""
    window, _ = wind
    checktab = window.central.checkWidget
    window.central.setCurrentWidget(checktab)
    checktab.fileInput.setText(ttorrent)
    checktab.searchInput.setText(dir1)
    checktab.checkButton.click()
    assert checktab.textEdit.toPlainText() != ""


def test_check_tab_input1(wind, dir1):
    """Test checker procedure."""
    window, _ = wind
    checktab = window.central.checkWidget
    window.central.setCurrentWidget(checktab)
    checktab.browseButton2.browse(dir1)
    assert checktab.searchInput.text() != ""


def test_check_tab_input_2(wind, dir1):
    """Test checker procedure."""
    window, _ = wind
    checktab = window.central.checkWidget
    window.central.setCurrentWidget(checktab)
    checktab.browseButton1.browse(dir1)
    assert checktab.fileInput.text() != ""


def test_check_tab4(wind):
    """Test checker procedure again."""
    window, _ = wind
    checktab = window.central.checkWidget
    window.central.setCurrentWidget(checktab)
    tree_widget = checktab.treeWidget
    assert tree_widget.invisibleRootItem() is not None


def test_clear_logtext(wind):
    """Test checker logTextEdit widget function."""
    window, _ = wind
    checktab = window.central.checkWidget
    window.central.setCurrentWidget(checktab)
    text_edit = checktab.textEdit
    text_edit.insertPlainText("sometext")
    text_edit.clear_data()
    assert text_edit.toPlainText() == ""


def test_checktab_tree(wind):
    """Check tree item counting functionality."""
    window, _ = wind
    checktab = window.central.checkWidget
    window.central.setCurrentWidget(checktab)
    tree = TreeWidget(parent=checktab)
    item = TreePieceItem(type=0, tree=tree)
    item.progbar = ProgressBar(parent=tree, size=1000000)
    item.count(100000000)
    assert item.counted == 1000000


@pytest.mark.parametrize("size", list(range(18, 20)))
@pytest.mark.parametrize("index", list(range(1, 7, 2)))
@pytest.mark.parametrize("version", [1, 2, 3])
@pytest.mark.parametrize("ext", [".mkv", ".rar", ".r00", ".mp3"])
def test_singlefile(size, ext, index, version, wind):
    """Test the singlefile for create and check tabs."""
    window, _ = wind
    createtab = window.central.createWidget
    checktab = window.central.checkWidget
    window.central.setCurrentWidget(checktab)
    testfile = str(tempfile(exp=size))
    tfile = testfile + ext
    os.rename(testfile, tfile)
    metafile = tfile + ".torrent"
    createtab.path_input.clear()
    createtab.output_input.clear()
    createtab.browse_file_button.browse(tfile)
    createtab.output_input.setText(metafile)
    createtab.piece_length.setCurrentIndex(index)
    btns = [createtab.v1button, createtab.v2button, createtab.hybridbutton]
    for i, btn in enumerate(btns):
        if i + 1 == version:
            btn.click()
            break
    createtab.submit_button.click()
    createtab.submit_button.join()
    checktab.fileInput.clear()
    checktab.searchInput.clear()
    checktab.fileInput.setText(metafile)
    checktab.searchInput.setText(tfile)
    checktab.checkButton.click()
    ptext = checktab.textEdit.toPlainText()
    assert "100%" in ptext
    rmpath(tfile, metafile)



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Magnet Widget containing Testing creation of Magnet URIs."""

import os
from pathlib import Path

from PySide6.QtCore import Qt
from PySide6.QtWidgets import (QFileDialog, QHBoxLayout, QLabel, QLineEdit,
                               QPushButton, QSpacerItem, QToolButton,
                               QVBoxLayout, QWidget)
from torrentfile import magnet


class MagnetWidget(QWidget):
    """Tab for creating magnet URL's and downloading torrentfiles from them."""

    def __init__(self, parent=None):
        """Initialize the widget form for creating magnet URI's from a metafile."""
        super().__init__(parent=parent)
        self.window = parent.window
        self.layout = QVBoxLayout()
        self.setLayout(self.layout)
        self.hlayout = QHBoxLayout()
        self.top_spacer = QSpacerItem(0, 50)
        self.bottom_spacer = QSpacerItem(0, 400)
        self.metafile_label = QLabel("Metafile", parent=self)
        self.metafile_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.metafile_input = QLineEdit(parent=self)
        self.metafile_input.setStyleSheet("QLineEdit {margin: 10px;}")
        self.output = QLineEdit(parent=self)
        self.output.setStyleSheet("QLineEdit {margin: 10px;}")
        self.file_button = MetafileButton(parent=self)
        self.submit_button = SubmitButton(parent=self)
        self.hlayout.addWidget(self.metafile_input)
        self.hlayout.addWidget(self.file_button)
        self.layout.addSpacerItem(self.top_spacer)
        self.layout.addWidget(self.metafile_label)
        self.layout.addLayout(self.hlayout)
        self.layout.addWidget(self.output)
        self.layout.addWidget(self.submit_button)
        self.layout.addSpacerItem(self.bottom_spacer)
        self.setAcceptDrops(True)

    def dragEnterEvent(self, event):
        """Drag enter event for widget."""
        if event.mimeData().hasUrls:
            self.urls = event.mimeData().urls()
            event.accept()
            return True
        return event.ignore()

    def dragMoveEvent(self, event):
        """Drag Move Event for widgit."""
        if event.mimeData().hasUrls:
            self.urls = event.mimeData().urls()
            event.accept()
            return True
        return event.ignore()

    def dropEvent(self, event):
        """Drag drop event for widgit."""
        urls = event.mimeData().urls()
        path = urls[0].toLocalFile()
        if os.path.exists(path):
            self.metafile_input.setText(path)
            self.submit_button.click()
            return True
        return False


class SubmitButton(QPushButton):
    """Submit current information to be processed into a megnet URI."""

    def __init__(self, parent=None):
        """Initialize the button for magnet creation."""
        super().__init__(parent=parent)
        self.widget = parent
        self.window = parent.window
        self.setText("Create Magnet")
        self.clicked.connect(self.magnet)

    def magnet(self):
        """Create a magnet URI from information contained in form."""
        fd = self.widget.metafile_input.text()
        if os.path.exists(fd):
            uri = magnet(fd)
            self.widget.output.setText(uri)


class MetafileButton(QToolButton):
    """
    Find a .torrent file in native file browser button actions.

    Perameters
    ----------
    parent : `QWidget`
        This widgets parent.
    """

    def __init__(self, parent=None):
        """Initialize the metafile button."""
        super().__init__(parent=parent)
        self.window = parent.window
        self.widget = parent
        self.setText("...")
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.pressed.connect(self.select_metafile)

    def select_metafile(self, filename=None):
        """Find metafile in file browser."""
        if not filename:   # pragma: nocover
            filename, _ = QFileDialog.getOpenFileName(
                parent=self,
                caption="Select '.torrent' file",
                dir=str(Path.home()),
                filter="*.torrent",
                selectedFilter=None,
            )
        self.widget.metafile_input.setText(filename)



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Module for the menu bar."""

import json
import os
import webbrowser

from PySide6.QtGui import QAction
from PySide6.QtWidgets import QInputDialog, QMenu, QMenuBar

from torrentfileQt.qss import dark_theme, light_theme


class Menu(QMenu):
    """Menu item for MenuBar widget."""

    def __init__(self, text, parent=None):
        """Constructor for Menu Widget."""
        super().__init__(text, parent=parent)
        self.menubar = parent
        self.txt = text
        font = self.font()
        self.setObjectName(text + "MenuObject")
        font.setPointSize(10)
        self.setFont(font)


class ProfileAction:
    """Store the name and action taken when this menu button is triggered.

    Parameters
    ----------
    name : `str`
        name of the profile and menubutton
    action : `QAction`
        Action associated with the name
    parent : `QMenu`
        the menu which holds the action
    """

    def __init__(self, name, action, parent):
        """Initialize ProfileAction class."""
        self.name = name
        self.action = action
        self.parent = parent
        self.action.triggered.connect(self.trigger)

    def trigger(self):
        """Fill the create tab with saved values in profile."""
        filename = os.path.join(self.parent.home, "profiles.json")
        with open(filename, "rt") as jsonfile:
            profiles = json.load(jsonfile)
        profile = profiles[self.name]
        tab = self.parent.window.central.createWidget
        tab.source_input.setText(profile["source"])
        tab.announce_input.insertPlainText("\n".join(profile["trackers"]))
        tab.web_seed_input.insertPlainText("\n".join(profile["web_seeds"]))
        if profile["version"] == 3:
            tab.hybridbutton.click()
        elif profile["version"] == 2:
            tab.v2button.click()
        else:
            tab.v1button.click()
        if profile["private"]:
            tab.private.click()
        if profile["piece_length"]:
            for i in range(tab.piece_length.count()):
                if tab.piece_length.itemData(i) == profile["piece_length"]:
                    tab.piece_length.setCurrentIndex(i)
                    break


class MenuBar(QMenuBar):
    """Main menu bar for top level menu of program."""

    def __init__(self, parent=None):
        """Constructor for top level widgets."""
        super().__init__(parent=parent)
        self.window = parent
        self.profile_actions = []
        self.home = os.path.join(os.path.expanduser("~"), ".torrentfileQt")
        self.profiles = os.path.join(self.home, "profiles.json")
        self.file_menu = Menu("File")
        self.help_menu = Menu("Help")
        self.profile_menu = Menu("Profiles")
        self.addMenu(self.file_menu)
        self.addMenu(self.profile_menu)
        self.addMenu(self.help_menu)
        self.actionExit = QAction(self.window)
        self.actionAbout = QAction(self.window)
        self.actionDocs = QAction(self.window)
        self.actionRepo = QAction(self.window)
        self.actionDarkTheme = QAction(self.window)
        self.actionLightTheme = QAction(self.window)
        self.actionAddProfile = QAction(self.window)
        self.actionDarkTheme.setText("Dark Theme")
        self.actionLightTheme.setText("Light Theme")
        self.actionRepo.setText("Github Repository")
        self.actionExit.setText("Exit")
        self.actionAbout.setText("About")
        self.actionDocs.setText("Documentation")
        self.actionAddProfile.setText("Add Profile")
        self.add_profile_actions()
        self.file_menu.addAction(self.actionExit)
        self.file_menu.addAction(self.actionLightTheme)
        self.file_menu.addAction(self.actionDarkTheme)
        self.help_menu.addAction(self.actionAbout)
        self.help_menu.addAction(self.actionDocs)
        self.help_menu.addAction(self.actionRepo)
        self.profile_menu.addAction(self.actionAddProfile)
        self.actionExit.triggered.connect(self.exit_app)
        self.actionAbout.triggered.connect(self.about_qt)
        self.actionDocs.triggered.connect(documentation)
        self.actionRepo.triggered.connect(repository)
        self.actionAddProfile.triggered.connect(self.add_profile)
        self.actionLightTheme.triggered.connect(self.light_theme)
        self.actionDarkTheme.triggered.connect(self.dark_theme)
        self.actionLightTheme.setObjectName("actionLightTheme")
        self.actionDarkTheme.setObjectName("actiondDarkTheme")
        self.actionDocs.setObjectName("actionDocs")
        self.actionAddProfile.setObjectName("actionAddProfile")
        self.actionExit.setObjectName("actionExit")
        self.actionAbout.setObjectName("actionAbout")

    def light_theme(self):
        """Change the GUI theme for the application."""
        self.window.setStyleSheet(light_theme)

    def dark_theme(self):
        """Change the GUI application to dark theme."""
        self.window.setStyleSheet(dark_theme)

    def add_profile_actions(self):
        """Add action class for each profile found in profiles."""
        if os.path.exists(self.home):
            if os.path.exists(self.profiles):
                with open(self.profiles, "rt") as jsonfile:
                    try:
                        profiles = json.load(jsonfile)
                    except json.JSONDecodeError:  # pragma: nocover
                        profiles = {}
                for profile in profiles:
                    action = QAction(self.window)
                    action.setText(profile)
                    profile_action = ProfileAction(profile, action, self)
                    self.profile_menu.addAction(action)
                    self.profile_actions.append(profile_action)

    def add_profile(self, name=None):
        """Add a profile."""
        if not os.path.exists(self.home):
            os.mkdir(self.home)
        if not name:  # pragma: nocover
            name, result = QInputDialog.getText(
                self, "Add Profile", "Profile Name"
            )
            if not result:
                return
        tab = self.window.central.createWidget
        source = tab.source_input.text()
        trackers = tab.announce_input.toPlainText().split("\n")
        webseeds = tab.web_seed_input.toPlainText().split("\n")
        piece_length_index = tab.piece_length.currentIndex()
        piece_length = tab.piece_length.itemData(piece_length_index)
        if tab.hybridbutton.isChecked():
            version = 3
        elif tab.v2button.isChecked():
            version = 2
        else:
            version = 1
        private = False
        if tab.private.isChecked():
            private = True
        attributes = {
            "version": version,
            "private": private,
            "piece_length": piece_length,
            "trackers": trackers,
            "web_seeds": webseeds,
            "source": source,
        }
        if not os.path.exists(self.profiles):
            with open(self.profiles, "wt") as jsonfile:
                json.dump({name: attributes}, jsonfile)
        else:
            with open(self.profiles, "rt") as jsonfile:
                try:
                    profiles = json.load(jsonfile)
                except json.JSONDecodeError:  # pragma: nocover
                    profiles = {}
            profiles[name] = attributes
            with open(self.profiles, "wt") as jsonfile:
                json.dump(profiles, jsonfile)

    def about_qt(self):
        """Open the about qt menu."""
        self.window.app.aboutQt()  # pragma: nocover

    def exit_app(self):
        """Close application."""
        self.parent().app.quit()  # pragma: nocover


def documentation():  # pragma: no cover
    """Open webbrowser to TorrentFileQt documentation."""
    webbrowser.open_new_tab("https://alexpdev.github.io/torrentfile")


def repository():  # pragma: no cover
    """Open webbrowser to GitHub Repo."""
    webbrowser.open_new_tab("https://github.com/alexpdev/torrentfileQt")



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Testing module for most of GUI."""
import json
import os
from pathlib import Path

import pytest

from tests import wind, rmpath


@pytest.fixture(params=[1, 2, 3])
def profiles(request):
    """Pytest fixture for profiles."""
    root = Path(__file__).parent / "PROFILEDIR"
    profile_file = root / "profiles.json"
    if not os.path.exists(root):
        os.mkdir(root)
    with open(profile_file, "wt") as jsonfile:
        profile = {
            "example": {
                "version": request.param,
                "piece_length": 1048576,
                "source": "EXAMPLE",
                "private": True,
                "trackers": ["https://example.net/announce"],
                "web_seeds": [""],
            }
        }
        json.dump(profile, jsonfile)
    yield root, profile_file
    rmpath(root)


@pytest.fixture
def menubar_profiles(wind, profiles):
    """Pytest fixture for menubar profiles."""
    window, _ = wind
    menubar = window.menubar
    menubar.home, menubar.profiles = profiles
    menubar.add_profile_actions()
    return menubar, window, profiles


def test_wind():
    """Test pytest fixture for widnow."""
    assert wind


def test_menubar_profiles(menubar_profiles):
    """Test menubar profiles menu exists."""
    menubar, _, _ = menubar_profiles
    assert "example" in [i.name for i in menubar.profile_actions]


def test_menubar_profiles_actions(menubar_profiles):
    """Test menubar profiles actions."""
    menubar, window, profiles = menubar_profiles
    _, profile = profiles
    profs = json.load(open(profile))
    action = [i for i in menubar.profile_actions if i.name == "example"][0]
    action.action.trigger()
    tab = window.central.createWidget
    assert tab.source_input.text() == profs["example"]["source"]


def test_add_profile_with_profiles(menubar_profiles):
    """Test adding a profile."""
    menubar, window, profiles = menubar_profiles
    tab = window.central.createWidget
    tab.source_input.setText("SOURCE")
    tab.announce_input.setPlainText("https://announce.net")
    menubar.add_profile(name="test")
    _, profile = profiles
    profs = json.load(open(profile))
    assert "test" in profs


def test_add_profile_meta_v2(menubar_profiles):
    """Test adding a profile with other meta versions."""
    menubar, window, profiles = menubar_profiles
    window.menubar.light_theme()
    tab = window.central.createWidget
    tab.v2button.click()
    window.change_theme("light_theme")
    tab.source_input.setText("SOURCE")
    tab.private.click()
    tab.announce_input.setPlainText("https://announce.net")
    menubar.add_profile(name="test1")
    _, profile = profiles
    profs = json.load(open(profile))
    assert "test1" in profs


def test_add_profile_meta_hybrid(menubar_profiles):
    """Test adding a profile with other meta versions."""
    menubar, window, profiles = menubar_profiles
    tab = window.central.createWidget
    tab.v1button.click()
    window.menubar.light_theme()
    tab.source_input.setText("SOURCE")
    tab.announce_input.setPlainText("https://announce.net")
    menubar.add_profile(name="test2")
    _, profile = profiles
    profs = json.load(open(profile))
    assert "test2" in profs


def test_add_profile_without_profiles(wind):
    """Test adding a profile."""
    window, _ = wind
    path = window.menubar.home
    alt = Path(__file__).parent / "alt"
    if os.path.exists(window.menubar.home):
        os.rename(path, alt)  # pragma: nocover
    tab = window.central.createWidget
    tab.source_input.setText("SOURCE")
    tab.announce_input.setPlainText("https://announce.net")
    window.menubar.add_profile(name="test")
    window.menubar.dark_theme()
    profiles = window.menubar.profiles
    profs = json.load(open(profiles))
    assert "test" in profs
    rmpath(path)
    if os.path.exists(alt):
        os.rename(alt, path)  # pragma: nocover



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Testing module for most of GUI."""

import pytest
from torrentfile import TorrentFile, TorrentFileHybrid, TorrentFileV2

from tests import dir2, dir3, rmpath, tempfile, ttorrent, wind, MockEvent


def test_fixture():
    """Test fixtures."""
    assert dir2 and tempfile and ttorrent and wind and dir3


def test_info_tab_select1(wind, ttorrent):
    """Test Info tab select1."""
    window, app = wind
    infotab = window.central.infoWidget
    app.processEvents()
    window.central.setCurrentWidget(infotab)
    button = infotab.selectButton
    button.selectTorrent(path=ttorrent)
    assert infotab.nameEdit.text() != ""


@pytest.mark.parametrize(
    "creator", [TorrentFile, TorrentFileV2, TorrentFileHybrid]
)
def test_infotab_select2(wind, dir2, creator):
    """Test getting info for single file torrent."""
    window, app = wind
    torrent = creator(path=dir2)
    del torrent.meta["created by"]
    del torrent.meta["creation date"]
    del torrent.meta["announce-list"]
    outfile, _ = torrent.write()
    infotab = window.central.infoWidget
    app.processEvents()
    window.central.setCurrentWidget(infotab)
    button = infotab.selectButton
    button.selectTorrent(path=outfile)
    name = torrent.meta["info"]["name"]
    assert infotab.nameEdit.text() == name
    rmpath(outfile)


@pytest.mark.parametrize(
    "creator", [TorrentFile, TorrentFileV2, TorrentFileHybrid]
)
@pytest.mark.parametrize("size", list(range(16, 22)))
def test_infotab_single(wind, creator, size):
    """Test getting info for single file torrent."""
    window, app = wind
    tfile = tempfile(exp=size)
    torrent = creator(path=tfile)
    del torrent.meta["created by"]
    del torrent.meta["creation date"]
    del torrent.meta["announce-list"]
    outfile, _ = torrent.write()
    infotab = window.central.infoWidget
    app.processEvents()
    window.central.setCurrentWidget(infotab)
    button = infotab.selectButton
    button.selectTorrent(path=outfile)
    name = torrent.meta["info"]["name"]
    assert infotab.nameEdit.text() == name
    rmpath(outfile)


@pytest.mark.parametrize(
    "creator", [TorrentFile, TorrentFileV2, TorrentFileHybrid]
)
def test_infotab_nested(wind, creator, dir3):
    """Test getting info for single file torrent."""
    window, app = wind
    torrent = creator(path=dir3)
    del torrent.meta["created by"]
    del torrent.meta["creation date"]
    del torrent.meta["announce-list"]
    outfile, _ = torrent.write()
    infotab = window.central.infoWidget
    app.processEvents()
    window.central.setCurrentWidget(infotab)
    button = infotab.selectButton
    button.selectTorrent(path=outfile)
    name = torrent.meta["info"]["name"]
    assert infotab.nameEdit.text() == name
    rmpath(outfile)


def test_info_accept_method(wind, ttorrent):
    """Test drag enter event on info widget."""
    window, app = wind
    info = window.central.infoWidget
    info.window.central.setCurrentWidget(info)
    app.processEvents()
    event = MockEvent(ttorrent)
    assert info.dragEnterEvent(event)
    event = MockEvent(None)
    assert not info.dragEnterEvent(event)


def test_info_move_event(wind, ttorrent):
    """Test move event on info widget."""
    window, app = wind
    info = window.central.infoWidget
    info.window.central.setCurrentWidget(info)
    app.processEvents()
    event = MockEvent(ttorrent)
    assert info.dragMoveEvent(event)
    event = MockEvent(None)
    assert not info.dragMoveEvent(event)


def test_info_drop_event(wind, ttorrent):
    """Test drop event on editor widget."""
    window, app = wind
    info = window.central.infoWidget
    info.window.central.setCurrentWidget(info)
    app.processEvents()
    event = MockEvent(ttorrent)
    assert info.dropEvent(event)


def test_info_drop_false(wind):
    """Test drop event on editor widget is false."""
    window, app = wind
    info = window.central.infoWidget
    info.window.central.setCurrentWidget(info)
    app.processEvents()
    event = MockEvent(None)
    assert not info.dropEvent(event)



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Testing module for most of GUI."""
from PySide6.QtWidgets import QApplication, QMainWindow

from tests import wind
from torrentfileQt import qss
from torrentfileQt.infoTab import denom
from torrentfileQt.window import TabWidget


def test_wind():
    """Test wind fixture."""
    assert wind


def test_denom():
    """Test denom function."""
    num = denom(50000000000)
    assert num == "50.0GB"


def test_denom_small():
    """Test denom function for small number."""
    num = denom(357)
    assert num == "357"


def test_window1(wind):
    """Test Main Window Functionality."""
    window, _ = wind
    assert window is not None


def test_window2(wind):
    """Test Window Functionality."""
    window, _ = wind
    assert isinstance(window, QMainWindow)


def test_app1(wind):
    """Test app subclass."""
    _, app = wind
    assert isinstance(app, QApplication)


def test_app2(wind):
    """Test app subclass instance attribute."""
    window, app = wind
    assert app is window.app


def test_qss():
    """For coverage reporting."""
    assert qss is not None


def test_window_menubar1(wind):
    """Test window Menubar widget."""
    window, _ = wind
    assert window.menubar is not None


def test_tab_widget(wind):
    """Test window Tab widget."""
    window, _ = wind
    tabwidget = window.central
    assert isinstance(tabwidget, TabWidget)



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 20** AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Setup package."""

import json

from setuptools import find_packages, setup


def get_info():
    """Gather information from package files."""
    info = json.load(open("package.json"))
    info["long_description"] = open("README.md").read()
    return info


INFO = get_info()


setup(
    url=INFO["url"],
    name=INFO["name"],
    author=INFO["author"],
    license=INFO["license"],
    version=INFO["version"],
    keywords=INFO["keywords"],
    include_package_data=True,
    author_email=INFO["email"],
    description=INFO["description"],
    long_description=INFO["long_description"],
    packages=find_packages(exclude=[".env"]),
    install_requires=["torrentfile", "pyben", "PySide6"],
    project_urls={"Source Code": "https://github.com/alexpdev/torrentfileQt"},
    long_description_content_type="text/markdown",
    entry_points={
        "console_scripts": "torrentfileQt = torrentfileQt.window:start"
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: Apache Software License",
    ],
)



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""Module for testing procedures on Check Tab."""

import pyben
import pytest

from tests import dir1, dir2, ttorrent, wind, MockEvent


def test_fixtures():
    """Test fixtures."""
    assert dir1 and dir2 and ttorrent and wind


@pytest.mark.parametrize("field", ["announce", "name", "private", "comment"])
def test_editor_torrent_loading(field, wind, ttorrent):
    """Testing editor widget functionality."""
    window, app = wind
    editor = window.central.editorWidget
    editor.window.central.setCurrentWidget(editor)
    editor.fileButton.browse(ttorrent)
    app.processEvents()
    fields = []
    for i in range(editor.table.rowCount()):
        fields.append(editor.table.item(i, 0).text())
    assert field in fields


def test_editor_torrent_saving(wind, ttorrent):
    """Testing editor widget saving functionality."""
    window, app = wind
    editor = window.central.editorWidget
    editor.window.central.setCurrentWidget(editor)
    app.processEvents()
    editor.fileButton.browse(ttorrent)
    for i in range(editor.table.rowCount()):
        item1 = editor.table.item(i, 0)
        item2 = editor.table.item(i, 1)
        if item1.text() == "announce":
            item2.setText("other")
            break
    app.processEvents()
    editor.button.click()
    meta = pyben.load(ttorrent)
    assert meta["announce"] == "other"


def test_editor_accept_method(wind, ttorrent):
    """Test drag enter event on editor widget."""
    window, app = wind
    editor = window.central.editorWidget
    editor.window.central.setCurrentWidget(editor)
    app.processEvents()
    event = MockEvent(ttorrent)
    assert editor.dragEnterEvent(event)
    event = MockEvent(None)
    assert not editor.dragEnterEvent(event)


def test_editor_move_event(wind, ttorrent):
    """Test move event on editor widget."""
    window, app = wind
    editor = window.central.editorWidget
    editor.window.central.setCurrentWidget(editor)
    app.processEvents()
    event = MockEvent(ttorrent)
    assert editor.dragMoveEvent(event)
    event = MockEvent(None)
    assert not editor.dragMoveEvent(event)


def test_editor_drop_event(wind, ttorrent):
    """Test drop event on editor widget."""
    window, app = wind
    editor = window.central.editorWidget
    editor.window.central.setCurrentWidget(editor)
    app.processEvents()
    event = MockEvent(ttorrent)
    assert editor.dropEvent(event)


def test_editor_drop_false(wind):
    """Test drop event on editor widget is false."""
    window, app = wind
    editor = window.central.editorWidget
    editor.window.central.setCurrentWidget(editor)
    app.processEvents()
    event = MockEvent(None)
    assert not editor.dropEvent(event)


def test_editor_table_fields(wind, ttorrent):
    """Test the edit fields of table widget."""
    window, app = wind
    editor = window.central.editorWidget
    editor.window.central.setCurrentWidget(editor)
    app.processEvents()
    table, found = editor.table, 0
    editor.line.setText(ttorrent)
    table.handleTorrent.emit(ttorrent)
    for i in range(table.rowCount()):
        if table.item(i, 0):
            txt = table.item(i, 0).text()
            if txt in ["httpseeds", "url-list", "announce-list"]:
                wig, found = table.cellWidget(i, 1), found + 1
                for url in ['url8', 'url9']:
                    wig.add_button.click()
                    wig.line_edit.setText(url)
                wig.combo.focusOutEvent(None)
                lst = [wig.combo.itemText(j) for j in range(wig.combo.count())]
                assert len([i for i in ['url8', 'url9'] if i in lst]) == 2
                wig.remove_button.click()
    editor.button.click()
    assert found == 3



#! /usr/bin/python3
# -*- coding: utf-8 -*-

##############################################################################
# Copyright 2020 AlexPDev
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
##############################################################################
"""
Tab Widget containing all controls for creating a new .torrent file.

User must provide the path to the directory containing the what the
.torrent file will be created from.
"""
import os
from pathlib import Path
from threading import Thread

from PySide6.QtCore import Qt
from PySide6.QtWidgets import (QCheckBox, QComboBox, QFileDialog, QGridLayout,
                               QHBoxLayout, QLabel, QLineEdit, QPlainTextEdit,
                               QPushButton, QRadioButton, QSpacerItem,
                               QToolButton, QWidget)
from torrentfile import TorrentFile, TorrentFileHybrid, TorrentFileV2
from torrentfile.utils import path_piece_length

from torrentfileQt.qss import pushButtonEdit


class CreateWidget(QWidget):
    """CreateWidget contains all controls for creating a new .torrent file.

    Args:
        QWidget (`QObject`): Parent class to CreateWidget.
    """

    def __init__(self, parent=None):
        """
        Constructor for Create Widget.

        Args:
            parent ([`QWidget`], optional): Parent Widget. Defaults to None.
        """
        super().__init__(parent=parent)
        self.content_dir = None
        self.outpath = None
        self.window = parent.window
        self.layout = QGridLayout()
        self.setLayout(self.layout)

        self.hlayout1 = QHBoxLayout()
        self.hlayout2 = QHBoxLayout()
        self.hlayout3 = QHBoxLayout()
        self.hlayout0 = QHBoxLayout()

        self.path_label = QLabel("Path: ", parent=self)
        self.output_label = QLabel("Save To: ", parent=self)
        self.version_label = QLabel("Meta Version: ", parent=self)
        self.comment_label = QLabel("Comment: ", parent=self)
        self.announce_label = QLabel("Trackers: ", parent=self)
        self.web_seed_label = QLabel("Web-Seeds: ")
        self.source_label = QLabel("Source: ", parent=self)
        self.piece_length_label = QLabel("Piece Size: ", parent=self)

        self.path_input = QLineEdit(parent=self)
        self.output_input = QLineEdit(parent=self)
        self.source_input = QLineEdit(parent=self)
        self.comment_input = QLineEdit(parent=self)

        self.announce_input = QPlainTextEdit(parent=self)
        self.web_seed_input = QPlainTextEdit(parent=self)
        self.piece_length = ComboBox.piece_length(parent=self)
        self.private = QCheckBox("Private", parent=self)
        self.submit_button = SubmitButton("Create Torrent", parent=self)
        self.browse_dir_button = BrowseDirButton(parent=self)
        self.browse_file_button = BrowseFileButton(parent=self)
        self.output_button = OutButton(parent=self)
        self.v1button = QRadioButton("v1 (default)", parent=self)
        self.v1button.setChecked(True)

        self.v2button = QRadioButton("v2", parent=self)
        self.hybridbutton = QRadioButton("v1+2 (hybrid)", parent=self)
        self.spacer1 = QSpacerItem(150, 0)
        self.spacer2 = QSpacerItem(70, 0)

        sizePolicy = self.path_input.sizePolicy()
        sizePolicy.setHorizontalPolicy(sizePolicy.MinimumExpanding)

        self.announce_label.setAlignment(Qt.AlignmentFlag.AlignRight)
        self.web_seed_label.setAlignment(Qt.AlignmentFlag.AlignRight)
        self.path_label.setAlignment(Qt.AlignmentFlag.AlignRight)
        self.path_input.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.path_input.setSizePolicy(sizePolicy)
        self.output_label.setAlignment(Qt.AlignmentFlag.AlignRight)
        self.output_input.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.version_label.setAlignment(Qt.AlignmentFlag.AlignRight)
        self.source_label.setAlignment(Qt.AlignmentFlag.AlignRight)
        self.source_input.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.comment_label.setAlignment(Qt.AlignmentFlag.AlignRight)
        self.comment_input.setAlignment(Qt.AlignmentFlag.AlignLeft)
        self.piece_length_label.setAlignment(Qt.AlignmentFlag.AlignRight)

        self.hlayout1.addWidget(self.path_input)
        self.hlayout0.addWidget(self.v1button)
        self.hlayout0.addWidget(self.v2button)
        self.hlayout0.addWidget(self.hybridbutton)
        self.hlayout1.addWidget(self.browse_dir_button)
        self.hlayout1.addWidget(self.browse_file_button)
        self.hlayout2.addWidget(self.piece_length)
        self.hlayout2.addItem(self.spacer1)
        self.hlayout2.addWidget(self.private)
        self.hlayout2.addItem(self.spacer2)
        self.hlayout3.addWidget(self.output_input)
        self.hlayout3.addWidget(self.output_button)

        self.layout.addWidget(self.path_label, 0, 0, 1, 1)
        self.layout.addLayout(self.hlayout1, 0, 1, 1, 3)
        self.layout.addWidget(self.output_label, 1, 0, 1, 1)
        self.layout.addLayout(self.hlayout3, 1, 1, 1, 3)
        self.layout.addWidget(self.source_label, 2, 0, 1, 1)
        self.layout.addWidget(self.source_input, 2, 1, 1, 3)
        self.layout.addWidget(self.comment_label, 3, 0, 1, 1)
        self.layout.addWidget(self.comment_input, 3, 1, 1, 3)
        self.layout.addWidget(self.announce_label, 4, 0, 1, 1)
        self.layout.addWidget(self.announce_input, 4, 1, 1, 3)
        self.layout.addWidget(self.web_seed_label, 5, 0, 1, 1)
        self.layout.addWidget(self.web_seed_input, 5, 1, 1, 3)
        self.layout.addWidget(self.piece_length_label, 6, 0, 1, 1)
        self.layout.addWidget(self.version_label, 7, 0, 1, 1)
        self.layout.addLayout(self.hlayout2, 6, 1, 1, 3)
        self.layout.addLayout(self.hlayout0, 7, 1, 1, 3)
        self.layout.addWidget(self.submit_button, 8, 0, 1, 4)
        for i in range(1, self.layout.columnCount()):
            self.layout.setColumnStretch(i, 1)
        self.layout.setObjectName("createWidget_formLayout")
        self.hlayout2.setObjectName("createWidget_hlayout2")
        self.submit_button.setObjectName("createWidget_submit_button")
        self.private.setObjectName("createWidget_private")
        self.path_label.setObjectName("createWidget_path_label")
        self.path_input.setObjectName("createWidget_path_input")
        self.piece_length.setObjectName("createWidget_piece_length")
        self.piece_length_label.setObjectName("createWidgetPiece_lengthLabel")
        self.source_label.setObjectName("createWidget_source_label")
        self.source_input.setObjectName("createWidget_source_input")
        self.announce_input.setObjectName("createWidget_announce_input")
        self.announce_label.setObjectName("createWidget_announce_label")
        self.comment_input.setObjectName("createWidget_comment_input")
        self.comment_label.setObjectName("createWidget_comment_label")
        self.browse_dir_button.setObjectName("createWidget_browsedir_button")
        self.browse_file_button.setObjectName("createWidget_browsefile_button")


def torrentfile_create(args, creator, name):  # pragma: no cover
    """Create new .torrent file in a seperate thread.

    Parameters
    ----------
    args : `dict`
        keyword arguements for the torrent creator.
    creator : `type`
        The procedure class for creating file.
    name : `list`
        container to add return values to.

    Returns
    -------
    `list`
        container for path to created torrent file.
    """
    torrent = creator(**args)
    outfile, _ = torrent.write()
    name.append(outfile)
    return name


class SubmitButton(QPushButton):
    """Button widget."""

    def __init__(self, text, parent=None):
        """Public Constructor for Submit Button.

        Parameters
        ----------
        text : `str`
            Text displayed on the button itself.
        parent : QWidget
            The tab widget parent.
        """
        super().__init__(text, parent=parent)
        self.thread = None
        self._text = text
        self.widget = parent
        self.window = parent.window
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.setText(text)
        self.clicked.connect(self.submit)

    def submit(self):
        """Submit Action performed when user presses Submit Button."""
        # Gather Information from other Widgets.
        args = {}
        if self.widget.private.isChecked():
            args["private"] = 1

        # add source to metadata
        sourcetext = self.widget.source_input.text()
        if sourcetext:
            args["source"] = sourcetext

        # add comments to metadata
        commenttext = self.widget.comment_input.text()
        if commenttext:
            args["comment"] = commenttext

        # at least 1 tracker input is required
        announce = self.widget.announce_input.toPlainText()
        announce = [i for i in announce.split("\n") if i]
        if announce:
            args["announce"] = announce

        # Calculates piece length if not specified by user.
        outtext = os.path.realpath(self.widget.output_input.text())
        if outtext:
            args["outfile"] = outtext

        current = self.widget.piece_length.currentIndex()
        if current:
            piece_length_index = self.widget.piece_length.currentIndex()
            piece_length = self.widget.piece_length.itemData(piece_length_index)
            args["piece_length"] = piece_length

        if self.widget.hybridbutton.isChecked():
            creator = TorrentFileHybrid
        elif self.widget.v2button.isChecked():
            creator = TorrentFileV2
        else:
            creator = TorrentFile

        path = self.widget.path_input.text()
        args["path"] = path
        name = []
        self.thread = Thread(
            group=None, target=torrentfile_create, args=(args, creator, name)
        )
        self.thread.start()
        return self

    def join(self):
        """Join the thread until it completes."""
        self.thread.join()


class OutButton(QToolButton):
    """Button widget."""

    def __init__(self, parent=None):
        """Constructor for file picker for outfile button."""
        super().__init__(parent=parent)
        self.window = parent.window
        self.widget = parent
        self.setText("...")
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.clicked.connect(self.output)

    def output(self, outpath=None):
        """Assign output path for created torrent file."""
        if not outpath:  # pragma: no cover
            outpath, _ = QFileDialog.getSaveFileName(
                parent=self,
                caption="Save as...",
                dir=str(Path.home()),
                filter="*.torrent",
                selectedFilter="",
            )
        if outpath:
            self.widget.output_input.clear()
            self.parent().output_input.setText(outpath)
            self.parent().outpath = outpath


class BrowseFileButton(QPushButton):
    """Button widget for browse."""

    def __init__(self, parent=None):
        """Public constructor for browsebutton class."""
        super().__init__(parent=parent)
        self.setText("Select File")
        self.window = parent
        self.setStyleSheet(pushButtonEdit)
        self.setCursor(Qt.CursorShape.PointingHandCursor)
        self.clicked.connect(self.browse)

    def browse(self, path=None):
        """Browse Action performed when user presses button.

        Opens File/Folder Dialog.

        Returns:
            str: Path to file or folder to include in torrent.
        """
        caption = "Select file..."
        if not path:  # pragma: no cover
            path, _ = QFileDialog.getOpenFileName(
                parent=self, caption=caption, dir=str(Path.home())
            )
        if path != "":
            path = os.path.normpath(path)
            self.window.path_input.clear()
            self.window.output_input.clear()
            self.window.path_input.setText(path)
            self.window.output_input.setText(path + ".torrent")
            piece_length = path_piece_length(path)
            if piece_length < (2**20):
                val = f"{piece_length//(2**10)}KB"
            else:
                val = f"{piece_length//(2**20)}MB"
            for i in range(self.window.piece_length.count()):
                if self.window.piece_length.itemText(i) == val:
                    self.window.piece_length.setCurrentIndex(i)
                    break


class BrowseDirButton(QPushButton):
    """Browse filesystem folders for path."""

    def __init__(self, parent=None):
        """Constructor for folder browser button."""
        super().__init__(parent=parent)
        self.setText("Select Folder")
        self.window = parent
        self.setStyleSheet(pushButtonEdit)
        self.clicked.connect(self.browse)
        self.setCursor(Qt.CursorShape.PointingHandCursor)

    def browse(self, path=None):
        """Browse action performed when user presses button.

        Opens File/Folder Dialog.

        Returns:
            str: Path to file or folder to include in torrent.
        """
        caption = "Select contents folder..."
        if not path:  # pragma: no cover
            path = QFileDialog.getExistingDirectory(
                parent=self, caption=caption, dir=str(Path.home())
            )
        if path:
            path = os.path.realpath(path)
            self.window.path_input.clear()
            self.window.output_input.clear()
            self.window.path_input.setText(path)
            self.window.output_input.insert(path + ".torrent")
            try:
                piece_length = path_piece_length(path)
            except PermissionError:  # pragma: no cover
                return
            if piece_length < (2**20):
                val = f"{piece_length//(2**10)}KB"
            else:  # pragma: no cover
                val = f"{piece_length//(2**20)}MB"
            for i in range(self.window.piece_length.count()):
                if self.window.piece_length.itemText(i) == val:
                    self.window.piece_length.setCurrentIndex(i)
                    break
        return


class ComboBox(QComboBox):
    """Combo box options for selecting piece length."""

    def __init__(self, parent=None):
        """Constructor for ComboBox."""
        super().__init__(parent=parent)
        self.addItem("")
        self.setEditable(False)
        self.profile_data = None

    @classmethod
    def piece_length(cls, parent=None):
        """Create a piece_length combobox."""
        box = cls(parent=parent)
        for exp in range(14, 26):
            if exp < 20:
                item = str((2**exp) // (2**10)) + "KB"
            else:
                item = str((2**exp) // (2**20)) + "MB"
            box.addItem(item, 2**exp)
        return box
